package modelo;

import java.util.List;
import java.util.LinkedList;


public class Jugador {
	
	private boolean humano;//false:ordenador, true:persona
	private int numJugador;
	private List<Carta> reparto;
	private String nombre;
	private int clasificado;
	
	public Jugador(){
		humano=false;
		numJugador=0;
		reparto= new LinkedList<Carta>();
		nombre="PC";
		clasificado = 0;
	}
	
	public Jugador(String n){
		humano=true;
		numJugador=0;
		reparto = new LinkedList<Carta>();
		nombre = n;
		clasificado = 0;
		
	}
	
	public Jugador(boolean humano, int numJugador){
		clasificado = 0;
		this.humano=humano;
		this.numJugador=numJugador;
		reparto=new LinkedList<Carta>();
		if(humano)
			nombre=new String("PFC");
		else
			nombre=new String("PC"+numJugador);
		this.numJugador=numJugador;
	}
	
	public String toString(){
		
		if(humano)
			return toStringGrafico();
		else
			return toStringNormal();
	}
	
	public String toStringNormal(){
		String s=new String("Jugador "+getNombre()+": ");

		if(reparto.size()==1)
			s+=reparto.get(0);
		else{
			for(int i=0;i<reparto.size()-1;i++)
				s+=reparto.get(i)+",";
			
			s+=reparto.get(reparto.size()-1);
			
		}
		return s;
	}

	
	public String toStringGrafico(){
		int l = this.size();
		int num;
		char palo;
		String s = new String();
		
		//Calculo del cuerpo de la carta
		for(int i=0; i<l;i++){
			s+="+---------+ ";
		}
		s+="\n";
		for(int i=0;i<l;i++){
			s+="|         | ";
		}
		s+="\n";
		
		for(int i=0;i<l;i++){
			num=(reparto.get(i)).getNumero();
			if(num<10)
				s+="|    "+num+"    | ";
			else
				if(num==10)
					s+="|   "+num+"    | ";
				else
					if(num==11)
						s+="|  Sota   | ";
					else
						if(num==12)
							s+="| Caballo | ";
						else
							s+="|   Rey   | ";
		}
		s+="\n";
		
		for(int i=0;i<l;i++){
			s+="|         | ";
		}
		s+="\n";
			
		for(int i=0;i<l;i++){
			palo = (reparto.get(i)).getPalo();
			if(palo=='o')
				s+="|   Oro   | ";
			else
				if(palo=='c')
					s+="|  Copas  | ";
				else
					if(palo=='e')
						s+="| Espadas | ";
					else
						s+="| Bastos  | ";
		}
		s+="\n";
		for(int i=0;i<l;i++){
			s+="+---------+ ";
		}
		s+="\n     ";
		for(int i=0;i<l;i++){
			s+=(i+1)+"           ";
		}
		s+="\n";
		
		
		return s;
	}
	

	//Roba n cartas de la Baraja baraja, con Baraja auxilizar mazo 
	public void roba(int n, Baraja baraja, Baraja mazo){
		if(baraja.size()==0 && mazo.size()==1){
			System.out.println("No hay ms cartas para robar");
		}
		else{
			if(n>=baraja.size()){
				Carta aux = mazo.remove(mazo.top());
				
				int rob=n-baraja.size();
				int siz = baraja.size();
				for(int i=0;i<siz;i++){
					reparto.add(baraja.remove());
				}
				
			
				baraja.addAll(mazo);
				baraja.barajar();
				
				mazo.clear();
				mazo.add(aux);
				
				roba(rob, baraja, mazo);
			}
			else{
				for(int i=0;i<n;i++)
					reparto.add(baraja.remove());
			}
		}
	}
	
	public void roba(Baraja baraja, Baraja mazo){
		this.roba(1, baraja, mazo);
	}
	
	//Dada una carta c (de robar) devuelve con una de sus cartas (carta 2 mejor que 1),  
	//sino devuelve null
	public Carta devuelve(Carta c){
		Carta dev =null;
		
		dev=devuelveFuerte(c);
		if(dev!=null){//He devuelto con un 2
			reparto.remove(dev);
		}
		else{
			dev=devuelveDebil(c);
			if(dev!=null)
				reparto.remove(dev);
		}
		return dev;
	}
	
	//Devuelve el ataque con un 2
	private Carta devuelveFuerte(Carta c){
		Carta dev = null;
		
		if(c.getNumero()==2){//Me hace robar por un 2
			boolean enc =false;
			int i=0;
			while(!enc && i<reparto.size()){
				Carta caux = reparto.get(i);
				if(caux.getNumero()==2){//Devuelvo con un 2
					enc=true;
					dev=caux;
				}
				else
					i++;
			}
		}
		else{//Me hace robar por un 1
			boolean enc=false;
			int i=0;
			while(!enc && i< reparto.size()){
				Carta caux = reparto.get(i);
				if(caux.getNumero()==2 && caux.getPalo()==c.getPalo()){
					enc=true;
					dev = caux;
				}
				else
					i++;
			}
		}		
		return dev;
	}
	
	
	private Carta devuelveDebil(Carta c){
		Carta dev = null;
		
		if(c.getNumero()==2){//me hace robar por un 2
			boolean enc =false;
			int i=0;
			while(!enc && i<reparto.size()){
				Carta caux = reparto.get(i);
				if(caux.getPalo()==c.getPalo() && caux.getNumero()==1){//Devuelve con un 1
					enc=true;
					dev=caux;
				}
				else
					i++;
			}
		}
		else{//Me hace robar por un 1
			boolean enc=false;
			int i=0;
			while(!enc && i< reparto.size()){
				Carta caux = reparto.get(i);
				if(caux.getNumero()==1){
					enc=true;
					dev = caux;
				}
				else
					i++;
			}
		}		
		return dev;
	}
	
	public Carta saca(Carta cmesa){
		List<Carta> cands = new LinkedList<Carta>();
		List<Carta> candsotas = new LinkedList<Carta>();
		Carta dev = null;
		
		
			for(int i=0;i<reparto.size();i++){
				Carta caux = reparto.get(i);
				if(caux.getNumero()==cmesa.getNumero() || caux.getPalo()==cmesa.getPalo() || caux.getNumero()==11){
					if(caux.getNumero()==13)//El rey tiene preferencia
						cands.add(0, caux);
					else{
						if(caux.getNumero()==11)
							candsotas.add(caux);
						else
							cands.add(caux);
					}
				}		
			}
			cands.addAll(candsotas);
		
		if(cands.size()!=0){
			dev=cands.remove(0);
			reparto.remove(dev);
		}
		return dev;
	}
	
	public Carta remove(Carta c){
		Carta caux = null;
		if(reparto.contains(c)){
			int i=reparto.indexOf(c);
			caux=reparto.get(i);
			reparto.remove(i);
		}
		return caux;
	}
	
	public int size(){
		return reparto.size();
	}
	
	public String getNombre(){
		return nombre;
	}
	public List<Carta> getReparto(){
		return reparto;
	}
	public boolean getHumano(){
		return humano;
	}
	
	public int getNumero(){
		return numJugador;
	}
	
	public boolean getClasificado(){
		return clasificado>0;
	}
	
	public void setClasificado(int clasif){
		clasificado = clasif;
	}
	
	public Carta getCarta(int i){
		Carta c = null;
		if(i>=0 && i<size())
			c = getReparto().get(i);
		
		return c;
	}
}