package modelo;

/*
 * 
 * 		---------------------------------------------------
 * 					Juego Pumba
 * 
 * 			Autor: Alejandro Raposo Lpez
 * 		--------------------------------------------------
 * 
 */

import java.util.LinkedList;
import java.util.List;

import vista.Clasificacion;
import vista.Mensaje;

public class Modelo {

	// Atributos
	public List<Jugador> jugadores; // Lista de jugadores en la partida
	public List<String> clasificacion; // Lista de jugadores en orden de
										// victoria
	public Baraja baraja; // Baraja de robar
	public Baraja mazo; // Baraja donde echar
	public int sig; // Siguiente jugador
	public int sentido; // 1: Hacia derecha, -1: Hacia izquierda
	public int numJugadores; // Numero de jugadores
	public int robar; // Cartas a robar actualmente

	public Jugador jugTurno; // Jugador del turno actual
	public Carta cmesa; // Ultima carta sacada (alto de la mesa)

	public Mensaje mensaje;

	// Juego por defecto, 4 jugadores
	public Modelo() {
		this(4);
	}

	// Juego para n jugadores
	public Modelo(int n) {

		mensaje = new Mensaje();

		sig = 0;
		sentido = 1;
		if (n < 2) {
			System.out.println("Debe haber al menos 2 jugadores.");
			System.out.println("Se establecer a 4 jugadores (por defecto).");
			numJugadores = 4;
		} else {
			if (n > 6) {
				System.out.println("El nmero mximo de jugadores es 6.");
				System.out.println("Se establecer 4 jugadores (por defecto).");
				numJugadores = 4;
			} else
				numJugadores = n;
		}
	}

	public void turno() {

		// Sucesion de turnos (mientras haya ms de un jugador)
		if (getNumJugadores() > 1) {
			cmesa = mazo.top();
			jugTurno = getJugador(sig);

			if (!jugTurno.getHumano()) {
				if (getRobar() > 0) {
					turnoRobarMaquina();
				} else {
					turnoNormalMaquina();
				}
			}

			if (jugTurno.size() == 0) {// Se qued sin cartas (sale de la
										// partida)

				numJugadores--;
				jugTurno.setClasificado(4 - numJugadores);
				clasificacion.add(new String(jugTurno.getNombre()));
				mensaje.setMensaje("Jugador " + jugTurno.getNombre() + " gana!");

				setSig();
			}
			mensaje.lanzar();

		} else {// Solo queda 1 jugador
			clasificacion.add(getJugador(sig).getNombre());

			Clasificacion clasi = new Clasificacion();
			clasi.clasificar(clasificacion);
			clasi.setModal(true);
			clasi.setVisible(true);
		}

	}

	// Establece el nombre, crea los jugadores por IA y reparte
	public void inicializa(String nombre) {

		robar = 0;

		clasificacion = new LinkedList<String>();

		// Inicializa jugadores
		jugadores = new LinkedList<Jugador>();

		if (nombre.equals(""))
			jugadores.add(new Jugador("PFC"));
		else
			jugadores.add(new Jugador(nombre));

		for (int i = 1; i < this.getNumJugadores(); i++)
			jugadores.add(new Jugador(false, i));

		// Crea la baraja, baraja, reparte y saca la primera
		baraja = new Baraja(false);// Baraja donde se roba
		baraja.barajar();
		mazo = new Baraja(true);// La baraja donde se echan cartas (vacía)
		mazo.add(baraja.remove());// Saca la primera carta

		// Reparte
		for (int i = 0; i < this.getNumJugadores(); i++) {
			getJugador(i).roba(7, baraja, mazo);
		}

		efecto(mazo.top().getNumero());

	}

	// --------------------------------------------------
	// ---------------- Metodos observadores ------------

	// Devuelve el jugador i-esimo
	public Jugador getJugador(int i) {
		return jugadores.get(i);
	}

	public Jugador getJugador(String s) {
		Jugador j = null;
		for (int i = 0; i < numJugadores; i++) {
			Jugador jaux = getJugador(i);
			if (jaux.getNombre().equals(s)) {
				j = jaux;
			}
		}

		return j;
	}

	// Devuelve la carta mas alta del mazo (la que esta encima de la mesa)
	public Carta getTop() {
		return mazo.top();
	}

	public int getSig() {
		return sig;
	}

	public int getSentido() {
		return sentido;
	}

	public int getNumJugadores() {
		return numJugadores;
	}

	public int getRobar() {
		return robar;
	}

	// -------------------------------------------------------

	// -------------------------------------------------------
	// ------------------ Metodos modificadores --------------

	public void setSig(int s) {
		this.sig = s;
	}

	// Avanza un turno segun el sentido
	public void setSig() {
		sig = sig + sentido;
		
		if(sig<0)
			sig = 3;
		
		if(sig>3)
			sig=0;

		if (getJugador(sig).getClasificado())
			setSig();
	}

	// Cambia de sentido
	public void setSentido() {
		sentido *= -1;
	}

	public void setNumJugadores(int n) {
		this.numJugadores = n;
	}

	public void setRobar(int r) {
		this.robar = r;
	}

	// -------------------------------------------------------

	private int jugadorMenosCartas() {
		int max = Integer.MAX_VALUE;
		int jug = 0;

		for (int i = 0; i < jugadores.size(); i++) {
			if (i != sig) {
				if (getJugador(i).size() < max
						&& !getJugador(i).getClasificado()) {
					jug = i;
					max = getJugador(i).size();
				}
			}
		}
		return jug;
	}

	// posible mejora estrategica: sacar otro rey en lugar de cartas de este
	// palo (si es mejor)
	private void estrategiaRey(Carta c) {
		String men = c.toString() + ": ";
		List<Carta> laux = jugTurno.getReparto();// Cartas del jugador
		List<Carta> cands1 = new LinkedList<Carta>();// Lista cartas sin efecto
		List<Carta> cands2 = new LinkedList<Carta>();// Lista cartas con efecto
		Carta caux;
		for (int i = 0; i < jugTurno.size(); i++) {
			caux = (Carta) laux.get(i);
			if (caux.getPalo() == c.getPalo()) {// Candidata
				if (caux.getNumero() > 2 && caux.getNumero() < 11
						&& caux.getNumero() != 7)// Cartas sin efecto
					cands1.add(caux);
				else
					// Cartas de efecto
					cands2.add(caux);
			}
		}

		if (cands1.size() + cands2.size() == 0) {// No tengo ninguna, robo

			Carta rob = null;
			jugTurno.roba(baraja, mazo);// Robo la primera vez
			rob = (Carta) jugTurno.getReparto().get(jugTurno.size() - 1);
			if (rob.getPalo() == c.getPalo() || rob.getNumero() == 13) {// Sirve?
				mazo.add(rob);
				jugTurno.remove(rob);
				if (rob.getNumero() == 13)
					estrategiaRey(rob);
				else {
					men.concat(rob.toString() + ".");
					efecto(rob.getNumero());
				}
			} else {// Aun no tengo
				
				jugTurno.roba(baraja, mazo);// Robo la segunda vez
				rob = (Carta) jugTurno.getReparto().get(jugTurno.size() - 1);
				if (rob.getPalo() == c.getPalo() || rob.getNumero() == 13) {// Sirve?
					mazo.add(rob);
					jugTurno.remove(rob);
					if (rob.getNumero() == 13)
						estrategiaRey(rob);
					else {
						men.concat(rob.toString() + ".");
						efecto(rob.getNumero());
					}

				} else {
					setSig();

				}
			}
		} else {// Tengo alguna (con o sin efecto)
			Carta cart = null;
			while (cands1.size() != 0) {
				cart = (Carta) cands1.remove(0);
				men.concat(cart.toString() + ".");
				mazo.add(cart);
				jugTurno.remove(cart);

			}
			if (cands2.size() != 0) {
				cart = (Carta) cands2.remove(0);
				men.concat(cart.toString() + ".");
				mazo.add(cart);
				jugTurno.remove(cart);
				efecto(cart.getNumero());

			} else
				setSig();
		}
		mensaje.setMensaje(men);
	}

	private void efecto(int numero) {
		switch (numero) {
		case 1:
			robar++;
			int sj = jugadorMenosCartas();
			System.out.println("Roba " + robar + " cartas el jugador "
					+ getJugador(sj).getNombre());
			setSig(sj);
			break;
		case 2:
			robar += 2;
			System.out.println("Roba " + robar + " cartas el siguiente");
			setSig();
			break;
		case 7:
			setSentido();
			System.out.println("Cambio de sentido");
			setSig();
			break;
		case 11:
			setSig();
			break;
		case 12:
			setSig();
			setSig();
			System.out.println("Te salto");
			break;
		default:
			setSig();
		}
	}

	public boolean esTurnoHumano(Jugador j) {
		return j.getHumano();
	}

	public void turnoNormalMaquina() {
		Carta dev = jugTurno.saca(cmesa);
		if (dev == null) {// NO tengo para sacar
			mensaje.setMensaje("Jugador " + jugTurno.getNombre() + ": Roba");

			jugTurno.roba(baraja, mazo);// Robo la primera
			dev = jugTurno.saca(cmesa);
			if (dev == null) {
				jugTurno.roba(baraja, mazo);// Robo la segunda
				dev = jugTurno.saca(cmesa);
			}
		}
		if (dev == null) {// Pasa el turno
			mensaje.setMensaje("Jugador " + jugTurno.getNombre()
					+ ": Ha robado 2 cartas y pasa");
			setSig();
		} else {// Tengo para echar
			mazo.add(dev);
			mensaje.setMensaje("Jugador " + jugTurno.getNombre() + ": " + dev);
			switch (dev.getNumero()) {
			case 1:
				robar++;
				int sj = jugadorMenosCartas();
				mensaje.setMensaje("Jugador " + jugTurno.getNombre()
						+ ": Roba " + robar + " cartas el jugador "
						+ getJugador(sj).getNombre());

				setSig(sj);
				break;
			case 2:
				robar += 2;
				mensaje.setMensaje("Jugador " + jugTurno.getNombre()
						+ ": Roba " + robar + " cartas el siguiente");
				setSig();
				break;
			case 7:
				setSentido();
				mensaje.setMensaje("Jugador " + jugTurno.getNombre() + ". "
						+ dev.toString() + ": Cambio de sentido");
				setSig();
				break;
			case 12:
				mensaje.setMensaje("Jugador " + jugTurno.getNombre() + ": "
						+ dev.toString() + ": Te salto");
				setSig();
				setSig();
				break;
			case 13:
				estrategiaRey(dev);
				break;
			default:
				setSig();
			}
		}

	}

	public void turnoRobarMaquina() {
		Carta dev = jugTurno.devuelve(cmesa);
		if (dev == null) {// No puedo devolver
			mensaje.setMensaje("Jugador " + jugTurno.getNombre() + ": Roba "
					+ robar + " cartas");

			jugTurno.roba(robar, baraja, mazo);
			robar = 0;
			if (cmesa.getNumero() == 2)// Por ser un 2 salta de turno
				setSig();
		} else {// Puedo devolver
			mazo.add(dev);
			int inc = dev.getNumero();
			robar += inc;
			int quien = jugadorMenosCartas();
			if (inc == 2)
				setSig();
			else
				setSig(quien);
			mensaje.setMensaje("Hace robar " + robar + " cartas al jugador "
					+ getJugador(sig).getNombre());

		}
	}
}